/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx;

import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.RuntimeErrorException;
import javax.management.RuntimeMBeanException;
import javax.management.RuntimeOperationsException;
import javax.naming.Context;

import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Provides a management representative to be used by JMX based management applications.
 * @author Adriana Danes
 */
public class ManagementReprImplJSR160 implements ManagementRepr {

    /**
     * JOnAS <code>logger</code>
     */
    private static Logger logger = null;
    /**
     * <code>mMBeanServerConnection</code> object representing the MBeanServer
     * on the server side.
     */
    private MBeanServerConnection mbeanServerConnection = null;
    /**
     * sever name
     */
    private String serverName = null;
    /**
     * ManagementRepr object constructor
     */
    protected ManagementReprImplJSR160() {
        logger = Log.getLogger("org.objectweb.jonas.jmx");
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG, "Management Representativ based on JSR160 created for jonasAdmin");
        }
    }

    /**
     * @param on The object name of the MBean to be checked
     * @return True if the MBean is already registered in the MBean server, false otherwise or if an exception is catched.
     */
    public boolean isRegistered(ObjectName on) {
        try {
            return mbeanServerConnection.isRegistered(on);
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * @param on The ObjectName of the MBean from which the attribute is to be retrieved.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @return The value of the attribute.
     * @throws ManagementException management operation failed
     */
    public Object getAttribute(ObjectName on, String attribute)
    throws ManagementException {
        // DEBUG
        if (mbeanServerConnection == null) {
            logger.log(BasicLevel.ERROR, "mbeanServerConnection is null");
            throw new ManagementException("mbeanServerConnection is null");
        }
        try {
            return mbeanServerConnection.getAttribute(on, attribute);
        } catch (Exception e) {
            logger.log(BasicLevel.WARN, "Error while getting attribute " + attribute + ". ObjectName = " + on);
            throw new ManagementException("Error while getting attribute. ", e);
        }
    }

    /**
     * @param on The ObjectName of the MBean of which attribute are to be retrieved.
     * @param attributes A list of the attributes to be retrieved.
     * @return Thelist of retrieved attributes.
     * @throws ManagementException management operation failed
     */
    public AttributeList getAttributes(ObjectName on, String[] attributes)
    throws ManagementException {

        try {
            return mbeanServerConnection.getAttributes(on, attributes);
        } catch (Exception e) {
            throw new ManagementException("Error while getting attributes: "
                    + e.getClass().getName(), e);
        }
    }

    /**
     * @param on The ObjectName of the MBean of which the attribute is to be set.
     * @param attribute A String specifying the name of the attribute to be retrieve.
     * @param value The value to set to the attribute.
     * @throws ManagementException management operation failed
     */
    public void setAttribute(ObjectName on, String attribute, Object value)
    throws ManagementException {
        if (logger.isLoggable(BasicLevel.DEBUG)) {
            logger.log(BasicLevel.DEBUG
                    , "Set Attribute called, on " + on.toString() + " to change attribute " + attribute
                    + " value to " + (String) value.toString());
        }

        try {
            mbeanServerConnection.setAttribute(on, new Attribute(attribute, value));
        } catch (Exception e) {
            throw new ManagementException("Error while setting attribute " + attribute + ": "
                    + e.getClass().getName(), e);
        }
    }

    /**
     * @param on The ObjectName of the MBean of which the attribute is to be set.
     * @param attributes A list of attributes: The identification of the attribute to be set and the value it is to be set to
     * @return The list of attributes that were set, with their new values.
     * @throws ManagementException management operation failed
     */
    public AttributeList setAttributes(ObjectName on, AttributeList attributes)
    throws ManagementException {

        try {
            return mbeanServerConnection.setAttributes(on, attributes);
        } catch (Exception e) {
            throw new ManagementException("Error while setting attributes: "
                    + e.getClass().getName(), e);
        }
    }

    /**
     * @param on The ObjectName of the MBean of which an operation is to be executed
     * @param operation name of the operation to execute
     * @param param arguments of the operation to execute
     * @param signature types of arguments
     * @return return the operation result
     * @throws ManagementException management operation failed
     */
    public Object invoke(ObjectName on, String operation, Object[] param, String[] signature)
    throws ManagementException {

        try {
            return mbeanServerConnection.invoke(on, operation, param, signature);
        } catch (Exception e) {
            String message = "";
            String targetExcName = null;
            Throwable exc = null;
            if (e instanceof MBeanException
                    || e instanceof ReflectionException
                    || e instanceof RuntimeMBeanException
                    || e instanceof RuntimeOperationsException
                    || e instanceof RuntimeErrorException) {

                Exception targetExc = null;
                if (e instanceof MBeanException)                {
                    targetExc = ((MBeanException) e).getTargetException();
                } else if (e instanceof ReflectionException) {
                    targetExc = ((ReflectionException) e).getTargetException();
                } else if (e instanceof RuntimeMBeanException) {
                    targetExc = ((RuntimeMBeanException) e).getTargetException();
                } else if (e instanceof RuntimeOperationsException) {
                    targetExc = ((RuntimeOperationsException) e).getTargetException();
                } else if (e instanceof RuntimeErrorException) {
                    Error atargetExc = ((RuntimeErrorException) e).getTargetError();
                    targetExc = new Exception(atargetExc.getMessage());
                }
                targetExcName = targetExc.toString();
                exc = targetExc;
            } else {
                exc = e;
            }
            if (logger.isLoggable(BasicLevel.DEBUG)) {
                logger.log(BasicLevel.DEBUG
                        , "Exception ----[ " + e.toString() + " while invoking operation " + operation
                        + " on MBean " + on.toString() + " ]-------");
                if (targetExcName != null) {
                    logger.log(BasicLevel.DEBUG, "-------[ Embedded error : ]-------");
                    logger.log(BasicLevel.DEBUG, "-------[ " + targetExcName + " ]-------");
                }
            }

            throw new ManagementException(message, exc);
        }
    }

    /**
     * @param on The ObjectName of the target MBean
     * @return A set containing the ObjectNames for the MBeans selected.
     * @throws ManagementException management operation failed
     */
    public java.util.Set queryNames(ObjectName on)
    throws ManagementException {
        try {
            return (java.util.Set) mbeanServerConnection.queryNames(on, null);
        } catch (Exception e) {
            throw new ManagementException("Error while getting MBeans names: " + e.getClass().getName()
                    , e);
        }
    }

    /**
     * @return An instance of MBeanInfo allowing the retrieval of all
     * attributes and operations of this MBean.
     * @throws ManagementException management operation failed
     */
    public MBeanInfo getMBeanInfo(ObjectName name)
    throws ManagementException {
        try {
            return (MBeanInfo) mbeanServerConnection.getMBeanInfo(name);
        } catch (Exception e) {
            throw new ManagementException("Error while getting MBean info: " + e.getClass().getName()
                    , e);
        }
    }

    /**
     * This method is no more necessary with JSR160 support
     * @return null
     * @throws javax.naming.NamingException not possible anymore
     */
    public Context getContext() throws javax.naming.NamingException {
        return null;
    }

    /**
     * This method is no more necessary with JSR160 support
     * @return a non null String
     */
    public String getCurrentRMIConnectorName() {
        return "jonas";
    }

    /**
     * This method is no more necessary with JSR160 support
     */
    public void setCurrentRMIConnectorName(String name) throws Exception {
        ;
    }

    /**
     * This method is no more necessary with JSR160 support
     */
    public void resetCurrentRMIConnectorName() {
        ;
    }

    /**
     * This method is no more necessary with JSR160 support
     * @return null
     * @throws javax.naming.NamingException not possible anymore
     */
    public Set getRMIConnectorsNames() throws javax.naming.NamingException {
        HashSet res = new HashSet();
        res.add("jonas");
        return res;
    }

    /**
     * This method is no more necessary with JSR160 support
     * @return null
     */
    public String getJonasNamingServiceURL() {
        return "url";
    }

    /**
     * This method is no more necessary with JSR160 support
     * @throws javax.naming.NamingException not possible anymore
     */
    public void setJonasNamingServiceURL(String url) throws javax.naming.NamingException {
        ;
    }

    /**
     * This method is no more necessary with JSR160 support
     * @throws javax.naming.NamingException not possible anymore
     */
    public void setNamingEnvCtx(Properties env) throws javax.naming.NamingException {
        ;
    }
    /**
     * @return Returns the mbeanServerConnection.
     */
    public MBeanServerConnection getMBeanServerConnection() {
        return mbeanServerConnection;
    }
    /**
     * @param mbeanServerConnection The mbeanServerConnection to set.
     */
    public void setMBeanServerConnection(
            MBeanServerConnection mbeanServerConnection) {
        this.mbeanServerConnection = mbeanServerConnection;
    }

    public String getServerName() {
        return serverName;
    }

    public void setServerName(String serverName) {
        this.serverName = serverName;
    }
}
